]> git.sesse.net Git - movit/blobdiff - effect.h
Make the blur use the resize functionality, which also unbreaks the in-between sampli...
[movit] / effect.h
index 082c234c5292ebf7361326022b6eb918d16c8365..c016a55070c2fb63585cd381c46c9e69a4de77a4 100644 (file)
--- a/effect.h
+++ b/effect.h
@@ -14,7 +14,9 @@
 #include <string>
 #include <vector>
 
-#include <GL/gl.h>
+#include <assert.h>
+
+#include "opengl.h"
 
 class EffectChain;
 
@@ -96,6 +98,21 @@ public:
        // needs mipmaps, you will also get them).
        virtual bool needs_mipmaps() const { return false; }
 
+       // Whether this effect wants to output to a different size than
+       // its input(s). If you set this to true, the output will be
+       // bounced to a texture (similarly to if the next effect set
+       // needs_texture_bounce()).
+       virtual bool changes_output_size() const { return false; }
+
+       // If changes_output_size() is true, you must implement this to tell
+       // the framework what output size you want.
+       //
+       // Note that it is explicitly allowed to change width and height
+       // from frame to frame; EffectChain will reallocate textures as needed.
+       virtual void get_output_size(unsigned *width, unsigned *height) const {
+               assert(false);
+       }
+
        // How many inputs this effect will take (a fixed number).
        // If you have only one input, it will be called INPUT() in GLSL;
        // if you have several, they will be INPUT1(), INPUT2(), and so on.
@@ -115,17 +132,19 @@ public:
        // Returns the GLSL fragment shader string for this effect.
        virtual std::string output_fragment_shader() = 0;
 
// Set all uniforms the shader needs in the current GL context.
// The default implementation sets one uniform per registered parameter.
        //
        // <sampler_num> is the first free texture sampler. If you want to use
        // textures, you can bind a texture to GL_TEXTURE0 + <sampler_num>,
        // and then increment the number (so that the next effect in the chain
        // will use a different sampler).
-       //
-       // NOTE: Currently this is also abused a bit to set other GL state
-       // the effect might need.
-       virtual void set_uniforms(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+       virtual void set_gl_state(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+
+       // If you set any special OpenGL state in set_gl_state(), you can clear it
+       // after rendering here. The default implementation does nothing.
+       virtual void clear_gl_state();
 
        // Set a parameter; intended to be called from user code.
        // Neither of these take ownership of the pointer.
@@ -115,17 +132,19 @@ public:
        virtual std::string output_fragment_shader() = 0;
 
-       // Set all uniforms the shader needs in the current GL context.
-       // The default implementation sets one uniform per registered parameter.
+       // Set all OpenGL state that this effect needs before rendering.
+       // The default implementation sets one uniform per registered parameter,
+       // but no other state.
        //
        // <sampler_num> is the first free texture sampler. If you want to use